@@ -18,6 +18,7 @@ import uuid
 
 from oslo_log import log as logging
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 import sqlalchemy as sa
 from sqlalchemy import orm
@@ -59,9 +60,6 @@ class VNFD(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
     # In future, single service VM may accomodate multiple services.
     service_types = orm.relationship('ServiceType', backref='vnfd')
 
-    # driver to create hosting vnf. e.g. noop, nova, heat, etc...
-    infra_driver = sa.Column(sa.String(255))
-
     # driver to communicate with service managment
     mgmt_driver = sa.Column(sa.String(255))
 
@@ -84,8 +82,7 @@ class ServiceType(model_base.BASE, models_v1.HasId, models_v1.HasTenant):
 class VNFDAttribute(model_base.BASE, models_v1.HasId):
     """Represents attributes necessary for spinning up VM in (key, value) pair
 
-    key value pair is adopted for being agnostic to actuall manager of VMs
-    like nova, heat or others. e.g. image-id, flavor-id for Nova.
+    key value pair is adopted for being agnostic to actuall manager of VMs.
     The interpretation is up to actual driver of hosting vnf.
     """
 
@@ -112,7 +109,7 @@ class VNF(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
     description = sa.Column(sa.Text, nullable=True)
 
     # sufficient information to uniquely identify hosting vnf.
-    # In case of service VM, it's UUID of nova VM.
+    # In case of openstack manager, it's UUID of heat stack.
     instance_id = sa.Column(sa.String(64), nullable=True)
 
     # For a management tool to talk to manage this hosting vnf.
@@ -131,8 +128,7 @@ class VNF(model_base.BASE, models_v1.HasId, models_v1.HasTenant,
 class VNFAttribute(model_base.BASE, models_v1.HasId):
     """Represents kwargs necessary for spinning up VM in (key, value) pair.
 
-    key value pair is adopted for being agnostic to actuall manager of VMs
-    like nova, heat or others. e.g. image-id, flavor-id for Nova.
+    key value pair is adopted for being agnostic to actuall manager of VMs.
     The interpretation is up to actual driver of hosting vnf.
     """
 
@@ -161,7 +157,9 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
 
     def _get_resource(self, context, model, id):
         try:
-            return self._get_by_id(context, model, id)
+            if uuidutils.is_uuid_like(id):
+                return self._get_by_id(context, model, id)
+            return self._get_by_name(context, model, id)
         except orm_exc.NoResultFound:
             if issubclass(model, VNFD):
                 raise vnfm.VNFDNotFound(vnfd_id=id)
@@ -176,8 +174,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         return dict((attr.key, attr.value) for attr in attributes_db)
 
     def _make_service_types_list(self, service_types):
-        return [{'id': service_type.id,
-                 'service_type': service_type.service_type}
+        return [service_type.service_type
                 for service_type in service_types]
 
     def _make_vnfd_dict(self, vnfd, fields=None):
@@ -187,8 +184,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                 vnfd.service_types)
         }
         key_list = ('id', 'tenant_id', 'name', 'description',
-                    'infra_driver', 'mgmt_driver',
-                    'created_at', 'updated_at')
+                    'mgmt_driver', 'created_at', 'updated_at')
         res.update((key, vnfd[key]) for key in key_list)
         return self._fields(res, fields)
 
@@ -209,10 +205,6 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         res.update((key, vnf_db[key]) for key in key_list)
         return self._fields(res, fields)
 
-    @staticmethod
-    def _infra_driver_name(vnf_dict):
-        return vnf_dict['vnfd']['infra_driver']
-
     @staticmethod
     def _mgmt_driver_name(vnf_dict):
         return vnf_dict['vnfd']['mgmt_driver']
@@ -225,9 +217,8 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         vnfd = vnfd['vnfd']
         LOG.debug(_('vnfd %s'), vnfd)
         tenant_id = self._get_tenant_id_for_create(context, vnfd)
-        infra_driver = vnfd.get('infra_driver')
-        mgmt_driver = vnfd.get('mgmt_driver')
         service_types = vnfd.get('service_types')
+        mgmt_driver = vnfd.get('mgmt_driver')
 
         if (not attributes.is_attr_set(service_types)):
             LOG.debug(_('service types unspecified'))
@@ -240,7 +231,6 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                 tenant_id=tenant_id,
                 name=vnfd.get('name'),
                 description=vnfd.get('description'),
-                infra_driver=infra_driver,
                 mgmt_driver=mgmt_driver)
             context.session.add(vnfd_db)
             for (key, value) in vnfd.get('attributes', {}).items():
@@ -267,7 +257,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
         self._cos_db_plg.create_event(
             context, res_id=vnfd_dict['id'],
             res_type=constants.RES_TYPE_VNFD,
-            res_state=constants.RES_EVT_VNFD_NA_STATE,
+            res_state=constants.RES_EVT_ONBOARDED,
             evt_type=constants.RES_EVT_CREATE,
             tstamp=vnfd_dict[constants.RES_EVT_CREATED_FLD])
         return vnfd_dict
@@ -283,7 +273,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
             self._cos_db_plg.create_event(
                 context, res_id=vnfd_dict['id'],
                 res_type=constants.RES_TYPE_VNFD,
-                res_state=constants.RES_EVT_VNFD_NA_STATE,
+                res_state=constants.RES_EVT_NA_STATE,
                 evt_type=constants.RES_EVT_UPDATE,
                 tstamp=vnfd_dict[constants.RES_EVT_UPDATED_FLD])
         return vnfd_dict
@@ -308,7 +298,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                 self._cos_db_plg.create_event(
                     context, res_id=vnfd_db['id'],
                     res_type=constants.RES_TYPE_VNFD,
-                    res_state=constants.RES_EVT_VNFD_NA_STATE,
+                    res_state=constants.RES_EVT_NA_STATE,
                     evt_type=constants.RES_EVT_DELETE,
                     tstamp=vnfd_db[constants.RES_EVT_DELETED_FLD])
             else:
@@ -442,7 +432,7 @@ class VNFMPluginDb(vnfm.VNFMPluginBase, db_base.CommonDbMixin):
                 res_type=constants.RES_TYPE_VNF,
                 res_state=new_status,
                 evt_type=constants.RES_EVT_CREATE,
-                tstamp=timeutils.utcnow(), details="VNF status updated")
+                tstamp=timeutils.utcnow(), details="VNF creation completed")
 
     def _get_vnf_db(self, context, vnf_id, current_statuses, new_status):
         try:
